// This file is part of the RecrutNid projet.
//
// (c) 2002, Free Software Foundation
//
// RecrutNid is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// RecrutNid is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with RecrutNid; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
// Petit programme de modlisation de la mobilisation chez la fourmi 
// Plus d'informations sur http://www.nojhan.fr.st/
// Auteur : Johann Dro <nojhan@ifrance.com>
// Date : 4 dcembre 2001
// version : 1.5

/////////////////////////// READER ///////////////////////////////

#include <iostream>

#include <fstream>
// ifstream, ofstream

#include <vector>
// vector<>

#include <string>
// string

#include <algorithm>
// count(), find()

#include <cstdio>
//sprintf()

#include "SharedFunc.h"
// SplitString()

using namespace std;

//***** DoParameters(vector<string> Parameters) *****//
vector< vector<string> > DoParameters(vector<string> Parameters)
{// Paramtres -> Sous-Ordres

    vector<string> aSubOrder;
    vector< vector<string> > SubOrders;

    int i;
    for (i=0; i<Parameters.size(); i++){

        // spare le paramtre en : Sous-Ordre + Variable [+ Options]
        SplitString(Parameters[i], "(=?)", aSubOrder);

        SubOrders.push_back(aSubOrder); // stocke les sous-ordres
        aSubOrder.clear();

    }// for (Parameters)

    return SubOrders;

}//***** DoParameters(vector<string> Parameters) *****//




//***** ReadFile(istream* InFile, string Mode, vector<string> aSubOrder) *****//
vector< vector<float> > ReadFile(ifstream &InFile, string Mode, vector<string> aSubOrder)
{// InFile -> InMat

    string aVariable;
    string aTestVariable;
    float aTestValue;;

    // ex:    NumberOf(isContact)    NumberOf(isContact=1)
    aVariable = aSubOrder[1];

    if(aSubOrder.size()==4){ // ex: NumberOf(isContact?isOut=1)
        aTestVariable=aSubOrder[2];
        aTestValue=StringToFloat(aSubOrder[3]);
    }

    if(aSubOrder.size()==5){ // ex: NumberOf(isContact=1?isOut=1)
        aTestVariable=aSubOrder[3];
        aTestValue=StringToFloat(aSubOrder[4]);
    }

    char a_Line[1000+1];
    string s_Line;
    vector<string> v_Line;

    vector<float> Values;
    vector< vector<float> > InMat;
    vector< vector<float> > TestMat;

    while (!InFile.eof()) {

        s_Line.erase(s_Line.begin(), s_Line.end());
        v_Line.clear();
        Values.clear();

        getline(InFile, s_Line, '\n'); // prend une ligne

        SplitString(s_Line, "\t", v_Line); // en vecteur

        if (v_Line.size()<1) { // si fausse ligne
            continue;} // reboucle

        if (v_Line[0]==aVariable) { // si ligne de la variable cherche
            v_Line.erase( v_Line.begin() ); // efface l'en-tte
            Values = StringToFloat(v_Line); // passe en float
            InMat.push_back(Values); // stocke dans la matrice
        }

        if(aSubOrder.size()==4 || aSubOrder.size()==5){
        if (v_Line[0]==aTestVariable) { // si ligne de la variable cherche
            v_Line.erase( v_Line.begin() ); // efface l'en-tte
            Values = StringToFloat(v_Line); // passe en float
            TestMat.push_back(Values); // stocke dans la matrice
        }}
    }// while (eof)


    if(aSubOrder.size()==4 || aSubOrder.size()==5){
    int i,j;
    for(i=0; i<TestMat.size(); i++){
        for(j=0; j<TestMat[i].size(); j++){
            if(TestMat[i][j]==aTestValue){
                InMat[i][j]=-99;
            }
        }//for(j)
    }//for(i)
    }

    if (Mode=="ByAnts") {
        InMat = Transpose(InMat);
        return InMat;
    }else{
        return InMat;
    }

}//***** ReadFile(istream* InFile, string Mode, vector<string> aSubOrder) *****//




//***** DoCalc(vector< vector<float> > InMat, vector<string> aSubOrder) *****//
vector<float> DoCalc(vector< vector<float> > InMat, vector<string> aSubOrder)
{
    vector<float> OutVector;

    string aType;
           aType = aSubOrder[0];

    float aValue;
           if (aSubOrder.size()==2){ // si 2 items, ex: NumberOf(isContact)
               aValue = 1; // 1 par dfaut
           }
           if (aSubOrder.size()==3) { // si 3 items, ex: NumberOf(isContact=1)
               aValue = StringToFloat(aSubOrder[2]); // passe le 3me item
           }

    int i;
    for (i=0; i<InMat.size(); i++) {

        if (aType=="Sum") {
            float aSum;
            vector< float > aVector;

            int j;
            for(j=0; j<InMat[i].size(); j++) {
                if(InMat[i][j]!=-99){ // ignore les valeurs -99
                    aVector.push_back(InMat[i][j]);
                }
            }

            aSum = Sum(aVector); // Somme
            OutVector.push_back( aSum );
        }//if (Sum)


        if (aType=="Mean") {
            float aMean;
            vector< float > aVector;

            int j;
            for(j=0; j<InMat[i].size(); j++) {
                if(InMat[i][j]!=-99){ // ignore les valeurs -99
                    aVector.push_back(InMat[i][j]);
                }
            }

            aMean = Mean(aVector); // moyenne
            OutVector.push_back( aMean );
        }//if (Moy)


        if (aType=="Std") {
            float aStd;
            vector< float > aVector;

            int j;
            for(j=0; j<InMat[i].size(); j++) {
                if(InMat[i][j]!=-99){ // ignore les valeurs -99
                    aVector.push_back(InMat[i][j]);
                }
            }

            aStd = Std(aVector); // cart-type
            OutVector.push_back( aStd );
        }//if (Std)


        if (aType=="NumberOf") {
            float aCount=0;
            //count(InMat[i].begin(), InMat[i].end(), aValue, aCount); // compte

            int j;
            for (j=0; j<InMat[i].size(); j++) {
                if (InMat[i][j]==aValue & InMat[i][j]!=-99){
                    aCount++;
                }
            }

            OutVector.push_back( aCount );
        }//if (NumberOf)


        if (aType=="NumberBeforeFirst") {
            int aPos=-99;
            /*
            vector<float>::iterator it_Pos;
            it_Pos = find(InMat[i].begin(), InMat[i].end(), aValue); // itrateur sur la position
            aPos = it_Pos - InMat[i].begin(); // diffrence pour passer en entier
            */

            int j;
            for (j=0; j<InMat[i].size(); j++) {
                if (InMat[i][j]==aValue){
                    aPos=j;
                    break;
                }
            }

            OutVector.push_back( aPos );
        }//if (NumberBeforeFirst)

    }//for (InMat[i])

    return OutVector;

}//***** DoCalc(vector< vector<float> > InMat, vector<string> aSubOrder) *****//




//***** Treat(InFileName, OutFileName, Mode, Parameters) *****//
void Treat(string InFileName, string OutFileName, string Mode, vector<string> Parameters)
{// InFile -> OutFile

    vector<float> aVector;
    vector< vector<string> > SubOrders;
    vector< vector<float> > InMat, OutMat;


    ///// LECTURE du fichier d'entre /////
    ifstream InFile;
    InFile.open(InFileName.c_str(), ios::in); // ouvre en lecture

    if ( InFile.is_open() ) {
        InFile.close();// pour repositionner la lecture de zero

        SubOrders = DoParameters( Parameters ); // dfini les sous-ordres


        // MODES :  ByAnts, ByTime
        if(Mode=="ByAnts" || Mode=="ByTime"){
            //cout << "    <i> Calcul des resultats, mode : [" << Mode << "]" << endl;
            // parcours les sous-ordres
            int i;
            for (i=0; i<SubOrders.size(); i++) {
                    //cout << "        [" << Parameters[i] << "]";
                InFile.open(InFileName.c_str(), ios::in); // ouvre en lecture
                InMat = ReadFile( InFile, Mode, SubOrders[i]); // extrait la matrice  lire
                InFile.close(); // ferme pour la prochaine ouverture

                aVector = DoCalc(InMat, SubOrders[i]); // fait les calculs

                OutMat.push_back(aVector); // stocke le vecteur rsultat
                aVector.clear();

                    //cout << " - ok" << endl;
                 }// for (SubOrders)
                    //cout << endl;
        }//if( ByAnts || ByTime )


        //MODES : SumUp
        if(Mode=="SumUp") {
             cout << "    <i> Calcul des resultats, mode : [" << Mode << "]" << endl;
            vector<string> aOrder;
            vector<float> aVec, OutVec;
            char TmpBuf[100];

            int i=0,j,k;
            while( i > -1 ) {// parcours les fourmis/temps
                InMat.clear();
                i++;

                aOrder.clear();// construction du sous-ordre
                aOrder.push_back("aOrder");
                //aOrder.push_back( itoa(i, TmpBuf, 10) ); // ANSI C ?
                sprintf(TmpBuf, "%d", i);// ANSI C !
                aOrder.push_back( TmpBuf );

                InFile.open(InFileName.c_str(), ios::in); // ouvre en lecture

                InMat=ReadFile( InFile, Mode, aOrder );

                InFile.close();

                if(InMat.size()==0){// si pas de concordance
                    i=-1; break;}

                OutVec.clear();aVec.clear();
                for (j=0; j<SubOrders.size(); j++){// parcours les sous-ordres

                    aOrder[0] = SubOrders[j][0];
                    aVec = DoCalc( Transpose(InMat) ,aOrder);

                    for (k=0; k<aVec.size(); k++){// ajoute les rsultats au vecteur final
                        OutVec.push_back( aVec[k] );
                    }

                }//for(j)

                OutMat.push_back(OutVec);

                    cout << "\r    " << i << " items";
            }//while( i!=0 )
                cout << endl;

        }//if( SumUp )


    }else{
        cout << "<!> Impossible d'ouvrir le fichier d'entree : " << InFileName << endl;
        return;
    }// if (is_open)



    ///// ECRITURE du fichier de sortie /////
    ofstream OutFile;
    OutFile.open(OutFileName.c_str(), ios::out | ios::app); // ouvre en ecriture en mode append

    if ( OutFile.is_open() ) {
        int i, j;

        // premire ligne : en-ttes : ("temps"|"num") "variable1" "variable2" ...
        OutFile << endl << Mode; // titre

        if(Mode=="ByAnts" || Mode=="ByTime"){
            //cout << "    <i> Ecriture des resultats" << endl;
            for (i=0; i<SubOrders.size(); i++) {
                OutFile << "\t" << Parameters[i]; // en-ttes
            }
        }

        if(Mode=="SumUp"){// isole et renomme les enttes des fichiers ByAnts/ByTime
            cout << "    <i> Ecriture des resultats" << endl;

            vector<string> ReadParameters;

            InFile.open(InFileName.c_str(), ios::in); // ouvre en lecture

             string s_Line;
             vector<string> v_Line;
             while (!InFile.eof()) {

                 s_Line.erase(s_Line.begin(), s_Line.end());
                 v_Line.clear();

                 getline(InFile, s_Line, '\n'); // prend une ligne

                 SplitString(s_Line, "\t", v_Line); // en vecteur

                 if (v_Line.size()<1) { // si fausse ligne
                     continue;} // reboucle

                 if ( v_Line[0]=="ByAnts" || v_Line[0]=="ByTime" ) { // si ligne de l'en-tte
                     // efface l'en-tte
                     v_Line.erase( v_Line.begin() );
                     break;
                 }


             }// while (eof)

            InFile.close();

            for (i=0; i<SubOrders.size(); i++) {
                for (j=0; j<v_Line.size(); j++){
                    OutFile << "\t" << Parameters[i] << "[";
                    OutFile << v_Line[j] << "]"; // en-ttes
                }
            }

        }//if(SumUp)

        // une ligne : (temps/num) valeur1 valeur2 ...
        if(Mode=="ByAnts" || Mode=="ByTime"){
            OutMat=Transpose(OutMat);  // transposition
        }

        for(i=0; i<OutMat.size(); i++) {// valeurs
            OutFile << endl << i+1;

            for(j=0; j<OutMat[i].size(); j++) {
                OutFile << "\t" << OutMat[i][j];//ecrit une valeur
            }//for(j)

            if (Mode=="SumUp"){
                cout << "\r        " << i+1 << " / " << OutMat.size() << " lignes";
            }
        }//for(i)

        if (Mode=="SumUp"){
            cout << endl;
        }

        OutFile.close();
    }else{
        cout << "<!> Impossible d'ouvrir le fichier de sortie : " << OutFileName << endl;
        return;
    }// if (is_open)

}//***** Treat(InFileName, OutFileName, Mode, Parameters) *****//

