// This file is part of the RecrutNid projet.
//
// (c) 2002, Free Software Foundation
//
// RecrutNid is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// RecrutNid is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with RecrutNid; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
// Petit programme de modlisation de la mobilisation chez la fourmi 
// Plus d'informations sur http://www.nojhan.fr.st/
// Auteur : Johann Dro <nojhan@ifrance.com>
// Date : 4 dcembre 2001
// version : 1.5

///////////////////////////// FONCTIONS PARTAGEE ///////////////////////////////

#include <cstdlib>
// rand()

#include <string>
// string

#include <vector>
// vector<>

#include <cmath>
// log(), sqrt()

using namespace std;




//***** FilledMatrix(int DimL, int DimC, float FillValue) *****//
vector< vector<int> > FilledMatrix(int DimL, int DimC, int FillValue)
{// cre une matrice de dimension DimL x DimC, remplie de FillValue

    vector<int> Vec;
    int i;
    for(i=0; i<DimC; i++){
        Vec.push_back(FillValue);
    }

    vector< vector<int> > Mat;
    for(i=0; i<DimL; i++){
        Mat.push_back(Vec);
    }

    return Mat;
}
//***** FilledMatrix(int DimL, int DimC, float FillValue) *****//




//***** Transpose(vector< vector<float> > aMat) *****//
vector< vector<float> > Transpose(vector< vector<float> > aMat)
{// transpose une matrice

    int iSize=aMat.size();
    int jSize=aMat[0].size();

    vector<float> aVector;
    vector< vector<float> > NewMat;

    int i, j;
    // parcours de la matrice en sens transpos
    for (j=0; j<jSize; j++) {
        for(i=0; i<iSize; i++) {
                aVector.push_back(aMat[i][j]);
        }//j

        NewMat.push_back(aVector);
        aVector.clear();
    }//i

    return NewMat;
}//***** Transpose(vector< vector<float> > aMat) *****//



//***** Sum(vector<float> aVector, int Begin=0, int During=0) *****//
float Sum(vector<float> aVector, int Begin=0, int During=0)
{

    if (During==0) {
        During = aVector.size() - Begin; // si passe pas le End : prend tout le vecteur  la suite
    }

    float aSum=0;

    int j;
    for (j=Begin; j<During; j++) {
        aSum = aSum + aVector[j]; // somme
    }//for (j)

    return aSum; // stocke dans la sortie

}//***** Sum(vector<float> aVector, int Begin=0, int During=0) *****//



//***** Mean(vector<float> aVector, int Begin=0, int During=0) *****//
float Mean(vector<float> aVector, int Begin=0, int During=0)
{

    if (During==0) {
        During = aVector.size() - Begin; // si passe pas le End : prend tout le vecteur  la suite
    }

    float aSum, aMean;

    aSum = Sum(aVector, Begin, During); // Somme
    aMean = aSum / (During - Begin); // Moyenne

    return aMean;
}//***** Mean(vector<float> aVector, int Begin=0, int During=0) *****//




//***** Std(vector<float> aVector, int Begin=0, int During=0) *****//
float Std(vector<float> aVector, int Begin=0, int During=0)
{

    if (During==0) {
        During = aVector.size() - Begin; // si passe pas le End : prend tout le vecteur  la suite
    }

    vector<float> Deviation;
    float aMean, aDev, aStd;

    aMean = Mean(aVector, Begin, During); // moyenne

    int j;
    for (j=Begin; j<During; j++) {
        aDev = aMean - aVector[j]; // carts
        Deviation.push_back(aDev*aDev);// carrs
    }//for (j)

    aStd = sqrt( Mean(Deviation, Begin, During) ); // moyenne des carrs des carts

    return aStd;

}//***** Std(vector<float> aVector, int Begin=0, int During=0) *****//




//***** StringToFloat(string aString) *****//
float StringToFloat(string aString)
{
    float aFloat;

    aFloat = atof( aString.c_str() );

    return aFloat;

}//***** StringToFloat(string aString) *****//



//***** StringToFloat(vector<string> VString) *****//
vector<float> StringToFloat(vector<string> VString)
{
    float aFloat;
    vector<float> VFloat;

    int i;
    for (i=0; i<VString.size(); i++) {
        aFloat = atof( VString[i].c_str() ); // transforme en float
        VFloat.push_back(aFloat);
    }
    return VFloat;
}//***** StringToFloat(vector<string> VString) *****//




//***** SplitString (string& text, string& separators, vector<string>& words) *****//
void SplitString (string text, string separators, vector<string> & words)
{
    int n = text.length();
    int start, stop;
 
    start = text.find_first_not_of(separators);
    while ((start >= 0) && (start < n)) {
       stop = text.find_first_of(separators, start);

       if ((stop < 0) || (stop > n)) stop = n;

       words.push_back(text.substr(start, stop - start));
       start = text.find_first_not_of(separators, stop+1);
   }
}//***** SplitString (string& text, string& separators, vector<string>& words) *****//




//***** rand_normal(Mean, Std) *****//
float rand_normal(float Mean, float Std)
{/* renvoie un nombre issue d'une distribution N(Mean, Std) */
// cf. : http://www.mathtools.net/index.html rubrique "generate random numbers"

	float x1, x2, w, y1;
	static float y2;
	static int use_last = 0;

	if (use_last)		        /* use value from previous call */
	{
		y1 = y2;
		use_last = 0;
	}
	else
	{
		do {
			x1 = 2* (double)rand()/RAND_MAX-1; // rand ds [0,1]
			x2 = 2* (double)rand()/RAND_MAX-1;
			w = x1 * x1 + x2 * x2;
		} while ( w >= 1.0 );

		w = sqrt( (-2.0 * log( w ) ) / w );
		y1 = x1 * w;
		y2 = x2 * w;
		use_last = 1;
	}
	return (float)(Mean + y1 * Std);

}//***** rand_normal(Mean, Std) *****//




//***** Xor(int cond1, int cond2) *****//
int Xor(int cond1, int cond2)
{ // OU exclusif

     if ( (cond1==0 && cond2>0) || (cond1>0 && cond2==0) ) {
         return 1;
     }else{
         return 0;
     }
}//***** Xor(int cond1, int cond2) *****//

