// This file is part of the RecrutNid projet.
//
// (c) 2002, Free Software Foundation
//
// RecrutNid is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// RecrutNid is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with RecrutNid; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
// Petit programme de modlisation de la mobilisation chez la fourmi 
// Plus d'informations sur http://www.nojhan.fr.st/
// Auteur : Johann Dro <nojhan@ifrance.com>
// Date : 4 dcembre 2001
// version : 1.5

///////////////////////////////// CLASSE NID ///////////////////////////////////

#include <iostream>
// cout, <<, >>

#include <fstream>
// ofstream

#include <cstdlib>
// min()

#include <vector>
// vector<>

#include <string>
// string

#include <cmath>
// floor()

#include "itsBaseAnt.h"
// class itsBaseAnt

#include "SharedFunc.h"
// Xor()

using namespace std;

class itsNest
{
    //*** PROPRIETES ***//
  public :
    int DimL, DimC; // dimensions du nid
    int TotalTurn; // nombre d'itrations
    int TotalAnts; // nb de fourmis
    int ThisTurn; // n d'itration

//  protected :
    vector<itsBaseAnt> CurAnts; // le vecteur de toutes les fourmis  t
    vector<itsBaseAnt> NextAnts; // le mme  t+1


    //*** METHODES ***//
  public :
    void SetDim(int aDimL, int aDimC); // initialisation des dimensions

    // Ajoute une fourmi
    void AddNewAnt(int aL, int aC,
                       float aVmin, float aVs, float aVa,
                       float aMotion,
                       int aisScout, int aisOut,
                       int aisContact,
                       int aisGivingTropha, int aisTakingTropha);// renvoie le n de la nouvelle fourmi
    void Simulate(int aTotalTurn, string aFileName); // lance le calcul sur le nid
    void WriteNest(string aFileName); // sauve le nid courant
    int EndTurn(int aTurn); // un tour de plus
    itsNest();// constructeur

  protected :

    // Tente de faire un trophallaxie entre deux fourmis
    int TryMeet(int Ant1, int Ant2);
    // tente de sortir la fourmi
    int TryGoOut(int AntNb);
    // tente de bouger
    int TryMove(int AntNb);

    void PutOut(int AntNb); // sort la fourmi avec le n AntNb
    void PutIn(int AntNb, int atLine, int atColumn); // rentre la fourmi...
    void ResetTempFlags(); // met les flags d'action  0

};//***** dclaration de classe *****//



//***** constructeur *****//
itsNest::itsNest()
{
    DimL = 0;
    DimC = 0;
    TotalAnts = 0;
    TotalTurn = 0;
    ThisTurn=0;
}//***** constructeur *****//



//***** SetDim(int aDimL, int aDimC) *****//
void itsNest::SetDim(int aDimL, int aDimC)
{
    DimL = aDimL;
    DimC = aDimC;
    TotalAnts = 0;
    TotalTurn = 0;
    ThisTurn = 0;
    CurAnts.clear();
    CurAnts.clear();
    NextAnts.clear();
    NextAnts.clear();
}//***** SetDim(int aDimL, int aDimC) *****//



//***** PutOut(int AntNb) *****//
void itsNest::PutOut(int AntNb)
{
    NextAnts[AntNb].GoOut();
}//***** PutOut(int AntNb) *****//



//***** PutIn(int AntNb, int atLine, int atColumn) *****//
void itsNest::PutIn(int AntNb, int atLine, int atColumn)
{
    NextAnts[AntNb].GoIn(atLine, atColumn);
}
//***** PutIn(int AntNb, int atLine, int atColumn) *****//



//***** EndTurn() *****//
int itsNest::EndTurn(int aTurn)
{
    CurAnts=NextAnts; // raffectation des vecteurs
    aTurn++; // un tour de plus

    return aTurn;
}//***** EndTurn() *****//



//***** ResetTempFlags() *****//
void itsNest::ResetTempFlags()
{
    int i;
    for (i=0; i<TotalAnts; i++) {
        NextAnts[i].ResetTempFlags();
        CurAnts[i].ResetTempFlags();
    }

}//***** ResetTempFlags() *****//



////////////////////////////////////////////////////////////////////////////////
//////////////////////////////// STRUCTURANTES /////////////////////////////////


//***** WriteNest(char* aFileName) *****//
void itsNest::WriteNest(string aFileName)
{
    ofstream aFile;
    aFile.open(aFileName.c_str(), ios::out | ios::app); // ouvre en ecriture en mode append

    if ( aFile.is_open() ) // si correctement ouvert
    {
        aFile << "ThisTurn\t" << ThisTurn << endl;
        aFile << "ToTalAnts\t" << TotalAnts << endl;
        aFile << "DimL\t" << DimL << endl;
        aFile << "DimC\t" << DimC << endl;

        int i;

        aFile << "Num";
        for (i=0; i<CurAnts.size(); i++) {aFile << '\t' << i+1;}
        aFile << endl;

        aFile << "L";
        for (i=0; i<CurAnts.size(); i++) {aFile << '\t' << CurAnts[i].L;}
        aFile << endl;

        aFile << "C";
        for (i=0; i<CurAnts.size(); i++) {aFile << '\t' << CurAnts[i].C;}
        aFile << endl;

        aFile << "Va";
        for (i=0; i<CurAnts.size(); i++) {aFile << '\t' << CurAnts[i].Va;}
        aFile << endl;

        aFile << "Vmin";
        for (i=0; i<CurAnts.size(); i++) {aFile << '\t' << CurAnts[i].Vmin;}
        aFile << endl;

        aFile << "Vs";
        for (i=0; i<CurAnts.size(); i++) {aFile << '\t' << CurAnts[i].Vs;}
        aFile << endl;

        aFile << "Motion";
        for (i=0; i<CurAnts.size(); i++) {aFile << '\t' << CurAnts[i].Motion;}
        aFile << endl;

        aFile << "isScout";
        for (i=0; i<CurAnts.size(); i++) {aFile << '\t' << CurAnts[i].isScout;}
        aFile << endl;

        aFile << "isOut";
        for (i=0; i<CurAnts.size(); i++) {aFile << '\t' << CurAnts[i].isOut;}
        aFile << endl;

        aFile << "isContact";
        for (i=0; i<CurAnts.size(); i++) {aFile << '\t' << CurAnts[i].isContact;}
        aFile << endl;

        aFile << "isGivingTropha";
        for (i=0; i<CurAnts.size(); i++) {aFile << '\t' << CurAnts[i].isGivingTropha;}
        aFile << endl;

        aFile << "isTakingTropha";
        for (i=0; i<CurAnts.size(); i++) {aFile << '\t' << CurAnts[i].isTakingTropha;}
        aFile << endl;

        aFile.close();
    }else{
        cout << "<!> Impossible d'ouvrir le fichier de sauvegarde : " << aFileName;
    }

}//***** WriteNest(char* aFileName) *****//



//***** AddNewAnt(float* pTab) *****//
void itsNest::AddNewAnt(int aL, int aC,
                       float aVmin, float aVs, float aVa,
                       float aMotion,
                       int aisScout, int aisOut,
                       int aisContact,
                       int aisGivingTropha, int aisTakingTropha)
{
    itsBaseAnt* pAnt;

    pAnt=new itsBaseAnt(aL, aC,
                        aVmin, aVs, aVa,
                        aMotion,
                        aisScout, aisOut,
                        aisContact,
                        aisGivingTropha, aisTakingTropha
                        );// valeurs donnes

    NextAnts.push_back(*pAnt); // ajoute au vecteur
    //CurAnts.push_back(*pAnt);
    TotalAnts++; // une de plus !

    delete pAnt; // supprime l'objet temporaire, car copie est dans le vecteur

}//***** AddNewAnt(float* pTab) *****//




////////////////////////////////////////////////////////////////////////////////
////////////////////////////////// LOGIQUES ////////////////////////////////////


//***** TryMeet(int AntNb1, int AntNb2) *****//
int itsNest::TryMeet(int AntNb1, int AntNb2)
{
    // test si dj dehors ou en contact ou en tropha
    if (CurAnts[AntNb1].isOut==1 || CurAnts[AntNb2].isOut==1 ||
        CurAnts[AntNb1].isContact==1 || CurAnts[AntNb2].isContact==1 ||
        CurAnts[AntNb1].isGivingTropha==1 || CurAnts[AntNb2].isGivingTropha==1 ||
        CurAnts[AntNb1].isTakingTropha==1 || CurAnts[AntNb2].isTakingTropha==1) return 0;

        float VolTropha;// vont faire une tropha


       if ( Xor(CurAnts[AntNb1].isScout, CurAnts[AntNb2].isScout) ) { // si l'un ou l'autre est scout
           int G_Nb;
           int R_Nb;

           if (CurAnts[AntNb1].isScout==1) { // si c'est la premire
               G_Nb = AntNb1; // donneuse
               R_Nb = AntNb2; // receveuse
           }else{ // si c'est la seconde
               G_Nb = AntNb2;
               R_Nb = AntNb1;
           }

            VolTropha = min(CurAnts[R_Nb].Vs - CurAnts[R_Nb].Va,
                            CurAnts[G_Nb].Va);// volume le plus grand possible

            if(VolTropha == 0) return 0;// si fourmi dj pleine

            if(TryGoOut(G_Nb)==1) {// si sortie possible (sortira au prochain tour)
                NextAnts[G_Nb]=CurAnts[G_Nb];// efface la sortie gnre par TryGoOut
                return 0;// empche la rencontre
            }

            NextAnts[G_Nb].GiveVol(VolTropha);
            NextAnts[R_Nb].TakeVol(VolTropha);// tropha D => R

            CurAnts[G_Nb].GiveVol(VolTropha);
            CurAnts[R_Nb].TakeVol(VolTropha);// tropha aussi, pour viter pb de chevauchement

            NextAnts[G_Nb].isContact=1;
            NextAnts[R_Nb].isContact=1; // toutes les deux ont un contact

            NextAnts[R_Nb].isScout=1; // la receveuse devient scout  son tour

// provisoire ?
            NextAnts[R_Nb].Motion = NextAnts[G_Nb].Motion; // changement de la mobilit
// provisoire ?

            //TryGoOut(G_Nb);// sortie donneuse
            //TryGoOut(R_Nb);// sortie receveuse
           return 1;
       }else{
           return 0;
       }//if(isScout)

}//***** TryMeet(int AntNb1, int AntNb2) *****//




//***** TryGoOut(int AntNb) *****//
int itsNest::TryGoOut(int AntNb)
{

    // tests fourmi pas dj dehors
    if (CurAnts[AntNb].isOut==1) return 0;

    float Aff; // affamement
    Aff = (CurAnts[AntNb].Vmin - CurAnts[AntNb].Va); // Vmin-Va

    if (CurAnts[AntNb].isScout==1 && CurAnts[AntNb].Va<=0) { // si scout vide
        NextAnts[AntNb].GoOut(); // sortie scout
        return 1;
    }else{
        return 0;
    }

/*
   if (CurAnts[AntNb].isScout==0) {
        NextAnts[AntNb].GoOut(); // sortie des non scouts
    }
*/

}//***** TryGoOut(int AntNb) *****//




//***** TryMove(int AntNb) *****//
int itsNest::TryMove(int AntNb)
{

    // tests validits
    if (CurAnts[AntNb].isOut==1 || CurAnts[AntNb].isContact==1) return 0;

    int aL, aC;
    int DeltaL, DeltaC; // diffrence ligne/colonne

        // hasard entre -Motion et Motion
        DeltaL = (int) ( rand() % (2*(int)CurAnts[AntNb].Motion + 1) - (int)CurAnts[AntNb].Motion );
        DeltaC = (int) ( rand() % (2*(int)CurAnts[AntNb].Motion + 1) - (int)CurAnts[AntNb].Motion );

        aL = CurAnts[AntNb].L + DeltaL;
        aC = CurAnts[AntNb].C + DeltaC; // Bouge virtuellement

        if ( (aC >= 1) && (aC <= DimC) ) { // si reste dans les bornes COLONNES
            NextAnts[AntNb].C = aC;
        }else{ // si sort des bornes
            NextAnts[AntNb].C = abs( DimC - aC ); // reboucle
        }

        if ( (aL >= 1) && (aL <= DimL) ) { // si reste dans les bornes LIGNES
            NextAnts[AntNb].L = aL;
        }else{ // si sort des bornes
            NextAnts[AntNb].L = NextAnts[AntNb].L; // bouge pas dans cette direction
        }

        return 1;

}//***** TryMove(int AntNb) *****//




//***** Simulate(int aTotalTurn, char* aFileName) *****//
void itsNest::Simulate(int aTotalTurn, string aFileName)
{
        //cout << "    <i> Calcul en cours" << endl;

    TotalTurn = aTotalTurn;

    int t,i,j,k;
    float Ans;
    // SIMULATIONS //
    for (t=0; t<TotalTurn; t++) {

        // FOURMI //
        for (i=0; i<=(TotalAnts-1); i++) {

            Ans= TryGoOut( i ); // test sortie

            if (Ans==1) continue; // si i sort, ne fait rien d'autre

            // VOISINE //
            int DiffL, DiffC; // distances relatives
            for (j=0; j<=(TotalAnts-1); j++) {
                if (i==j) continue;

                // si voisine
                DiffL = abs(CurAnts[i].L - CurAnts[j].L); // distance Ligne
                DiffC = abs(CurAnts[i].C - CurAnts[j].C); // distance colonne

                if (DiffL<=1 && DiffC<=1) { // si  1 prs

                    Ans= TryMeet( i, j ); // tests rencontre
                    if(Ans==1) break; // si en rencontre une, sort boucle, ne fait pas d'autre rencontres

                } // if( Diff<=1 )

            }//for(j)
            // VOISINE //

            if (Ans==1) continue; // si a fait rencontre, ne bouge pas

            Ans= TryMove( i ); // bouge la fourmi en cours
            //if(Ans==1)

        }//for(i)
        // FOURMI //

        ThisTurn = EndTurn(ThisTurn);

        WriteNest(aFileName);

        ResetTempFlags();

            //cout << "\r        " << t+1 << " / " << TotalTurn;
    }//for(t)
    // SIMULATIONS //

        //cout << endl;

}//***** Simulate(int aTotalTurn, char* aFileName) *****//
