// This file is part of the RecrutNid projet.
//
// (c) 2002, Free Software Foundation
//
// RecrutNid is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// RecrutNid is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with RecrutNid; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
// Petit programme de modlisation de la mobilisation chez la fourmi 
// Plus d'informations sur http://www.nojhan.fr.st/
// Auteur : Johann Dro <nojhan@ifrance.com>
// Date : 4 dcembre 2001
// version : 1.5

/////////////////////////////// CLASSE SCRIPTE /////////////////////////////////
#include <iostream>

#include <string>
// string

#include <vector>
// vector<>

#include <fstream>
// ifstream

#include <cstdlib>
// rand()

#include <cstdio>
// remove()

#include "itsBaseAnt.h"
// class itsBaseAnt

#include "itsNest.h"
/// itsNest

#include "Reader.h"
// fonctions Reader

#include "Shower.h"
// fonctions Shower

#include "SharedFunc.h"
// SplitString() et rand_normal()

using namespace std;

class itsScript
{
    //*** PROPRIETES ***//
    public :
        string ScriptFileName; // nom du fichier script en cours
        string SaveFileName; // Fichier de sauvegarde
        string InFileName; // fichier d'entree
        itsNest TheNest;
        //itsReader TheReader;

    protected :
        vector<string> TheScript; // tout le script

        string CurLine; // Ligne
        string CurOrder; // instruction
        vector<string> CurParameters; // paramtres non interprts
        vector<float> CurValues; // paramtres interprts

        vector<float> CurAntVars; // values de la fourmi en cours
        vector<string> CurAntParameters; // param de la fourmi en cours

    //*** METHODES ***//
    public :
        void LoadScript(string aFileName); // charge le fichier script
        void DoScript(); // Execute le script

    protected :
            //void ReadNextLine(); // isole la prochaine ligne
            string ReadOrder(string aCurLine); // isole l'instruction
            vector<string> ReadParameters(string aCurLine); // isole les paramtres

            vector<float> DoParameters(vector<string> aCurParameters); // calcul des paramtres

            //void DoOrders();
            int DoOrders(int iLine); // Execute les instructions

}; //***** dclaration classe itsScript *****//




//***** LoadScript(string aFileName) *****//
void itsScript::LoadScript(string aFileName)
{// charge le fichier et formate les lignes dans un vecteur

    string StringLine;
    char CharLine[1000+1];

    ifstream aFile; // dclare l'objet fichier
    aFile.open(aFileName.c_str(), ios::in); // ouvre en lecture

    if (aFile.is_open()) { // si correctement ouvert
          cout << endl << "<i> Chargement du script : " << aFileName << endl;

       int i=0, j=0;
       int aPos;
       while(!aFile.eof()){ // parcours le fichier

           // prend la ligne
           aFile.getline(CharLine, sizeof(CharLine));
               cout << "\r    " << ++i << " lignes";

           // Supprime les lignes vides
           if (CharLine[0]=='\0'){ // si pas de ligne
              continue;} // reboucle

           StringLine = CharLine; // donc ligne non vide

           // Supprime les blancs et ignore les commentaires
           aPos=-1;
           while (1) {
               aPos= StringLine.find_first_of( "# \t", aPos+1 ); // si commentaire, espaces ou tabulation
               if( StringLine[aPos]=='#' ) { // si dbut de commentaire, ignore le reste
                   StringLine.erase( aPos, (StringLine.length()-aPos+1) ); // supprime tout ce qui suit
                   break;
               }
               if( aPos<0 || aPos>StringLine.length() ) {
                   break; // si pas d'espaces, sort
               }

               StringLine.erase(aPos,1); // supprime
           }// while (1)

           if (StringLine.length()==0) { // si ne reste rien aprs tout a
               continue;}

           // Stocke
           TheScript.push_back(StringLine); // vecteur de lignes correctement formates
           j++; // lignes correctes

       }//while(!eof)

       ScriptFileName = aFileName;
       aFile.close();
           cout << endl << "    Script charge : " << j << " instructions" << endl;
    }else{
        cout << endl << "<!> Impossible d'ouvrir le script :" << endl;
        cout << "    " << aFileName << endl;
    }// if (is_open)

}//***** LoadScript(string aFileName) *****//




//***** DoScript() *****//
void itsScript::DoScript()
{// Execute le script
 
int g_Details=1; // niveau de dtails de l'affichage

    int i=0;
    if (g_Details >= 1){
        cout << endl << "<i> execution du script" << endl;
    }
    //for (i=0; i < TheScript.size(); i++) { // parcours les lignes d'instructions
    while(i<TheScript.size()) {

        CurLine = TheScript[i];

        CurOrder = ReadOrder(CurLine);
        CurParameters = ReadParameters(CurLine);
        CurValues = DoParameters(CurParameters);

        //DoOrders();
        i=DoOrders(i);
    }
    if (g_Details >= 1){
        cout << endl << "<i> execution terminee" << endl;
    }
}//***** DoScript() *****//




//***** ReadOrder() *****//
string itsScript::ReadOrder(string aCurLine)
{// prend l'instruction
    string aCurOrder;
    aCurOrder="";

    int aPos=0;
    aCurOrder=aCurLine;

    // isole la partie instruction
    aPos=aCurOrder.find_first_of(":\n");  // si signe : ou fin de ligne
    aCurOrder.erase( aPos, (aCurOrder.length()-aPos+1) ); // supprime tout ce qui suit

    return aCurOrder;
}//***** ReadOrder() *****//




//***** ReadParameters() *****//
vector<string> itsScript::ReadParameters(string aCurLine)
{// met les paramtres non traduits dans un vecteur
    vector<string> aCurParameters;
    aCurParameters.clear();

    int aPos=0;
    string aParameters=aCurLine;

    // isole la partie paramtre
    aPos=aParameters.find_first_of(":");  // signe : ou fin de ligne

    if ( aPos<0 || aPos>aParameters.length() ){ // si pas de signe "="
        aParameters=""; // paramtres vides
        return aCurParameters;
    }

    aParameters.erase( 0, aPos+1 ); // supprime tout ce qui prcde

    // transforme en vecteur
    SplitString(aParameters, ",", aCurParameters); // spare les lments par virgule ou fin de ligne

    return aCurParameters;
}//***** ReadParameters() *****//




//***** DoParameters() *****//
vector<float> itsScript::DoParameters(vector<string> aCurParameters)
{// traduit les paramtres en nombres
    vector<float> aCurValues;
    aCurValues.clear();

    vector<string> TwoValues;
    float aValue;

    int i, aPos;
    for (i=0; i<aCurParameters.size(); i++) { // parcours chaque paramtres

        // hasard uniforme
        aPos = aCurParameters[i].find("-");
        if ( aPos>=0 && aPos<=aCurParameters[i].length() ) { // si trouve le tiret
            SplitString(aCurParameters[i], "-", TwoValues);
            // tire entre v[0] et v[1]
            aValue = ((float)rand()/RAND_MAX)
                     * atof( TwoValues[1].c_str() ) + atof( TwoValues[0].c_str() );
            aCurValues.push_back( aValue );
            TwoValues.clear();
            continue;
        }

        // hasard gaussien
        aPos = aCurParameters[i].find("/");
        if ( aPos>=0 && aPos<=aCurParameters[i].length() ) { // si trouve le slash
            SplitString(aCurParameters[i], "/", TwoValues);
            aValue = rand_normal(
                                 (float)atof( TwoValues[0].c_str() ),
                                 (float)atof( TwoValues[1].c_str() )
                                ); // tirage loi normale
            aCurValues.push_back( aValue );
            TwoValues.clear();
            continue;
        }

        // valeur seule
        aValue = atof( aCurParameters[i].c_str() ); // conversion
        aCurValues.push_back( aValue );
    }// for (i)
    return aCurValues;
}//***** DoParameters() *****//




//***** DoOrders() *****//
//void itsScript::DoOrders()
int itsScript::DoOrders(int iLine)
// return iLine++
{// appelle les fonctions correspondantes aux ordres en leur passant les paramtres

        if (CurOrder== "SetOutFile" ) { // Fichier de sauvegarde
            //cout << "    Out : [" << CurParameters[0] << "]" << endl;
            SaveFileName = CurParameters[0]; // sous forme de string
            //remove(SaveFileName.c_str()); // efface le fichier
            return ++iLine;}

        if (CurOrder== "SetInFile" ) { // Fichier de sauvegarde
            //cout << "    In : [" << CurParameters[0] << "]" << endl; //<< " - efface]"
            InFileName = CurParameters[0]; // sous forme de string
            //remove(InFileName.c_str()); // efface le fichier
            return ++iLine;}

        if (CurOrder== "MakeNest" ) {
            //cout << "    Nid : " << CurValues[0] << " x " << CurValues[1] << endl;
            TheNest.SetDim((int)CurValues[0], (int)CurValues[1]); // dimensions du nid
            return ++iLine;}

        if (CurOrder== "MakeAnt" ) {
                //cout << "    Fourmi : ";
                //copy(CurParameters.begin(), CurParameters.end(), ostream_iterator<string>(cout, " ") );
                //cout << endl;
            CurAntVars = CurValues; // vecteurs
            CurAntParameters = CurParameters;
            return ++iLine;}

        if (CurOrder== "LoadAnts" ) {
            int i, aTotal;
            aTotal=(int)CurValues[0];
                //cout << "    <i> Chargement des fourmis" << endl;
            for (i=0; i<aTotal; i++) {
                CurAntVars = DoParameters(CurAntParameters);

                TheNest.AddNewAnt((int)CurAntVars[0], (int)CurAntVars[1],
                                  (float)CurAntVars[2], (float)CurAntVars[3], (float)CurAntVars[4],
                                  (float)CurAntVars[5],
                                  (int)CurAntVars[6], (int)CurAntVars[7],
                                  (int)CurAntVars[8], (int)CurAntVars[9], (int)CurAntVars[10]
                                 );
                     //cout << "\r        " << i+1 << " / " << aTotal;
            }

            //cout << "    Total : " << TheNest.TotalAnts;

            TheNest.EndTurn(TheNest.ThisTurn);
            //TheNest.WriteNest(SaveFileName);

            //cout << endl;
            return ++iLine;
        }

        if (CurOrder== "Simulate" ) {
            TheNest.Simulate((int)CurValues[0], SaveFileName); // int, string
            return ++iLine;}

        if (CurOrder== "CalcByAnts") {
            string Mode = "ByAnts";// lance traitement des rsultats
            Treat(InFileName, SaveFileName, Mode, CurParameters);
            return ++iLine;}

        if (CurOrder== "CalcByTime") {
            string Mode = "ByTime";// lance traitement des rsultats
            Treat(InFileName, SaveFileName, Mode, CurParameters);
            return ++iLine;}

        if (CurOrder== "NestMaps") {
            MakeShow(InFileName, SaveFileName, CurValues);//cartes des nids
            return ++iLine;}

        if (CurOrder== "Loop") {

            cout << "    ---- Starting Loop : " << CurValues[0] << " ----" << endl;

            float MaxLoop = CurValues[0]; // positionne le nombre de boucles  faire

            string anOrder;
            int i, PosEndLoop;
            for(i=iLine; i<TheScript.size(); i++){ // trouve la position du "EndLoop"

                anOrder =TheScript[i];

                anOrder=ReadOrder( anOrder );

                if ( anOrder=="EndLoop" ) {
                    PosEndLoop=i;
                }
            }//for(i)

            for (i=0; i<MaxLoop; i++) { // effectue le nombre de boucle
                    cout << "\r    " << i+1 << " Loops";
                    //cout << "\r    -- Entering Loop : " << i << " --" << endl;


                // re-parcours les lignes d'instructions
                int j=iLine+1; // commence  ligne suivant le "Loop"
                while( j <= PosEndLoop ) {

                    CurLine = TheScript[j];

                    CurOrder = ReadOrder(CurLine);
                    CurParameters = ReadParameters(CurLine);
                    CurValues = DoParameters(CurParameters);

                    j=DoOrders(j); // rcursivite
                }// while(j)

            }///for(i)

                cout << endl << "    ---- EndLoop ----" << endl;

            return PosEndLoop;//++EndLoop
        }//if("Loop")

        if (CurOrder== "EndLoop") {
            return ++iLine;} // rien de particulier, juste une etiquette pour "Loop"

        if(CurOrder=="SumUp") {
            string Mode = "SumUp";

            Treat(InFileName, SaveFileName, Mode, CurParameters); // fonction de rsum

            return ++iLine;
        }

        if (CurOrder== "Delete") {
                //cout << "    Fichiers effaces : ";
            int i;
            for(i=0; i<CurParameters.size(); i++ ){
                remove( CurParameters[i].c_str() ); // supprime le fichier
                    //cout << "[" << CurParameters[i] << "] ";
            }
                //cout << endl;
            return ++iLine;
        }

        if (CurOrder== "Pause") {
                cout << "    - Script en pause -" << endl;
            system("PAUSE");
            return ++iLine;
        }

        cout << endl << "<!> impossible de reconnaitre cette instruction : " << endl;
        cout << "    " << CurOrder << endl;
        cout << "    lors de l'execution de : " << endl;
        cout << "    " << CurLine << endl;
        return ++iLine;

}//***** DoOrders() *****//
